-- local BBBRaidFrame = CreateFrame("Frame", "BBBRaid", UIParent);

-- init local variables --
local IconsBar = BBBRaidIconsFrame;
local _ = _;

--BBBRaid = {unpack(default_config)}; -- create default config
local defaults = {
   ["lock"] = 0,
   ["scale"] = 0.8,
   ["cd"] = 5,
   ["display"] = {
	  ["solo"] = 0,
	  ["instance"] = 0,
	  ["party"] = 0,
	  ["raid"] = 1,
   },
   ["show"] = {
	  ["icon"] = true,
	  ["flare"] = true,
	  ["countdown"] = true,
   },
}

-- init frames --
function InitBBBRaidMarker(self)
   self:RegisterEvent("PLAYER_LOGIN");
   self:RegisterEvent("GROUP_ROSTER_UPDATE");
   self:SetScript("OnEvent", onEvent);
   self:SetUserPlaced(true);
end

function InitIconsFrame(self) 
   local IconsBar = BBBRaidIconsFrame;
   local lastIcon = nil;
   for i = 1, 8 do
	  local icon = CreateFrame("Button", "BBBRaidIconButton"..i, IconsBar, "BBBRaidIconButtonTemplate");
	  icon:SetID(i);
	  if (lastIcon == nil) then
		 icon:SetPoint("LEFT", IconsBar, "LEFT", 5, 0);
	  else
		 icon:SetPoint("LEFT", lastIcon, "RIGHT");
	  end
	  icon:SetNormalTexture("interface\\targetingframe\\ui-raidtargetingicon_"..i)

	  lastIcon = icon;
   end
end

function InitFlaresFrame(self) 
   local bar = BBBRaidFlaresFrame;
   local lastIcon = nil;
   local iconList = {6,4,3,7,1};
   for i = 1, 6 do
	  local icon = CreateFrame("Button", "BBBRaidFlareButton"..i, bar, "BBBRaidFlareButtonTemplate");
	  icon:SetID(i);
	  if (lastIcon == nil) then
		 icon:SetPoint("LEFT", bar, "LEFT", 5, 0);
	  else
		 icon:SetPoint("LEFT", lastIcon, "RIGHT");
	  end
	  if (i==6) then
		 icon:SetNormalTexture("interface\\buttons\\ui-grouploot-pass-up");
	  else
		 icon:SetNormalTexture("interface\\targetingframe\\ui-raidtargetingicon_"..iconList[i]);
	  end
	  icon:SetAttribute("macrotext", "/click CompactRaidFrameManagerDisplayFrameLeaderOptionsRaidWorldMarkerButton\n/click DropDownList1Button"..i)

	  lastIcon = icon;
   end
   
   self:SetScript("OnShow",
				  function(self)
					 if (BBBRaidControlFrame) then
						BBBRaidControlFrame:ClearAllPoints();
						BBBRaidControlFrame:SetPoint("LEFT", self, "RIGHT");
					 end
				  end);
   self:SetScript("OnHide",
				  function(self)
					 if (BBBRaidControlFrame) then
						BBBRaidControlFrame:ClearAllPoints();
						BBBRaidControlFrame:SetPoint("TOPLEFT", BBBRaidMarker, "TOPLEFT");
					 end
				  end);
end

function InitMoverButton(self)
   self:SetScript("OnMouseDown",
				  function(self,button)
					 if (button=="LeftButton") then
						self:GetParent():StartMoving();
					 end
				  end);
   
   self:SetScript("OnMouseUp",
				  function(self)
					 self:GetParent():StopMovingOrSizing();
					 -- _, _, _, BBBRaid.frame.xOffset, BBBRaid.frame.yOffset = self:GetParent():GetPoint();
				  end);
   local button = CreateFrame("Button", "BBBRaidTogglerMenuButton", self);
   button:SetHeight(25);
   button:SetWidth(25);
   button:SetPoint("BOTTOMLEFT", self, "BOTTOMLEFT", 2, 0);
   button:SetNormalTexture("interface\\buttons\\ui-spellbookicon-prevpage-up");
   button:SetScript("OnClick",
					function(self) 
					   ToggleDropDownMenu(1, nil, BBBRaidBarToggleMenu, "BBBRaidTogglerMenuButton", -140, 70) 
					end);
end

function InitBarTogglerMenu(self) 
   local names = {"Icon", "Flare", "CountDown"}
   local bars = {};
   UIDropDownMenu_Initialize(self,
							 function(self, level)
								local info;
								for k,v in ipairs(names) do
								   info = UIDropDownMenu_CreateInfo();
								   info.text = v;
								   info.value = v:lower();
								   info.func = ToggleShow;
								   if (BBBRaid) then
									  info.checked = BBBRaid.show[v:lower()];
								   end
								   --info.checked = _G['BBB_TOGGLE_CHECKED_'..v:upper()];
								   UIDropDownMenu_AddButton(info, level);
								end
							 end);
end
   
function InitCountdownButton(self)
   local countDown = BBBRaidControlButton2:GetText();
   local type = self:GetText();
   -- print(type);
   if (type == 'D') then
	  self:SetAttribute("macrotext", "/dbm pull "..countDown)
   elseif (type == 'B') then
	  self:SetAttribute("macrotext", "/pull "..countDown)
   end
   
end

function InitCountDownMenu(self)
   local CountDownMenuItems = {"3", "5", "8", "10", "30"};
   local current = BBBRaidControlButton2:GetText();
   local key = 0;
   UIDropDownMenu_Initialize(self, 
							 function(self, level)
								local info;
								for k,v in ipairs(CountDownMenuItems) do
								   if (v == current) then key = k end; -- save the key for checked valued
								   info = UIDropDownMenu_CreateInfo();
								   info.text = v;
								   info.value = v;
								   info.func = ChangeCountDownSecs;
								   UIDropDownMenu_AddButton(info, level);
								end
							 end);
   UIDropDownMenu_SetSelectedID(self, key);
end

-- On function --

function ClickIconButton(self)
   SetRaidTargetIcon("target", self:GetID());
end

function ChangeCountDownSecs(self)
   if (InCombatLockdown()) then
	  print("Can't do this in combat");
	  return;
   else
	  UIDropDownMenu_SetSelectedID(BBBRaidCountDownMenu, self:GetID());
	  
	  local new = self:GetText();
	  BBBRaid.cd = new;
	  BBBRaidControlButton2:SetText(new);
	  BBBRaidDBMCountDownButton:SetAttribute("macrotext", "/dbm pull "..new);
	  BBBRaidBWCountDownButton:SetAttribute("macrotext", "/pull "..new);
   end
end

function ToggleShow(self)
   local barnames = {BBBRaidIconsFrame, BBBRaidFlaresFrame, BBBRaidControlFrame};
   BBBRaid.show[self.value] = not BBBRaid.show[self.value];
   if (self.checked) then
	  barnames[self:GetID()]:Hide();
   else
	  barnames[self:GetID()]:Show();
   end
end

function onEvent(self, event, addon, ...)
   if (event == "PLAYER_LOGIN") then
	  -- init addon setting
	  if not BBBRaid then
		 BBBRaid = copyDefaults(defaults);
	  end
		 -- print(BBBRaid.scale);
	  BBBRaidMarker:SetScale(BBBRaid.scale);
	  -- BBBRaidMarker:SetPoint("CENTER", UIParent, "CENTER", BBBRaid.frame.xOffset, BBBRaid.frame.yOffset);
	  if (BBBRaid.lock > 0) then
		 BBBRaidMover:Hide();
	  else
		 BBBRaidMover:Show();
	  end
	  updateDisplay();
	  updateShow();
	  
   elseif (event == "GROUP_ROSTER_UPDATE") then
	  updateDisplay()
   end
end

-- tool function

-- This function copies values from one table into another:
function copyDefaults(src, dst)
   -- If no source (defaults) is specified, return an empty table:
   if type(src) ~= "table" then return {} end
   -- If no target (saved variable) is specified, create a new table:
   if type(dst) then dst = {} end
   -- Loop through the source (defaults):
   for k, v in pairs(src) do
	  -- If the value is a sub-table:
	  if type(v) == "table" then
		 -- Recursively call the function:
		 dst[k] = copyDefaults(v, dst[k])
		 -- Or if the default value type doesn't match the existing value type:
	  elseif type(v) ~= type(dst[k]) then
		 -- Overwrite the existing value with the default one:
		 dst[k] = v
	  end
   end
   -- Return the destination table:
   return dst
end

-- to get current display setting 
function getDisplay() 
   if (IsInGroup()) then
	  if(IsInRaid()) then
		 if (UnitIsGroupLeader("player") or UnitIsGroupAssistant("player")) then
			return BBBRaid.display.raid;
		 else
			return 0;
		 end
	  else
		 return BBBRaid.display.party;
	  end
   else 
	  return BBBRaid.display.solo;
   end
end

-- toggle show/hide
function toggleVisible(frame)
   if (frame:IsVisible()) then
	  frame:Hide();
   else
	  frame:Show();
   end
end

-- show/hide Frame by getDisplay()
function updateDisplay()
   if (getDisplay() > 0) then
	  BBBRaidMarker:Show();
   else
	  BBBRaidMarker:Hide();
   end
end

-- update bar display by BBBRaid.show
function updateShow()
   local barTables= {
	  icon = BBBRaidIconsFrame,
	  flare = BBBRaidFlaresFrame,
	  countdown = BBBRaidControlFrame};
   for k,v in pairs(BBBRaid.show) do
	  if (v) then
		 barTables[k]:Show();
	  else
		 barTables[k]:Hide();
	  end
   end
end

-- define slash command functions

local function help()
   print(
[[|cffff7d0aBBB|rRaidMarker Usage:
/|cffff7d0abrm|r scale [n]
/|cffff7d0abrm|r lock||unlock
/|cffff7d0abrm|r reset
/|cffff7d0abrm|r test
/|cffff7d0abrm|r mode
/|cffff7d0abrm|r solo/party/raid [show||hide]
]]);
end

local function showMode()
   local color_text = {"|cff00ff00show|r"};
   color_text[0]="|cffff3300hide|r";
   print( string.format(
[[Current Display Mode:
solo => %s
party => %s
raid => %s
]], color_text[BBBRaid.display.solo],
--    color_text[BBBRaid.display.instance],
	color_text[BBBRaid.display.party],
	color_text[BBBRaid.display.raid]));
end

SLASH_BBBRAID1 = "/bbbraidmarker";
SLASH_BBBRAID2 = "/bbb";
SLASH_BBBRAID3 = "/brm";

SlashCmdList["BBBRAID"] = function(args)
   local command, arg1 = strsplit(" ", args, 2);
   local main_frame = BBBRaidMarker;
   command = command:lower();
   local mode_list = {solo=1, party=1, raid=1};
   if (command=="scale") then local size = tonumber(arg1);
	  if (type(size) == 'number') then
		 BBBRaidMarker:SetScale(size);
		 BBBRaid.scale = size;
	  end
   elseif (command=="unlock") then
	  BBBRaid.lock = 0;
	  BBBRaidMover:Show();
   elseif (command=="lock") then
	  BBBRaid.lock = 1;
	  BBBRaidMover:Hide();
   elseif (command=="test") then
	  toggleVisible(BBBRaidMarker);
   elseif (command=="reset") then
	  BBBRaid = copyDefaults(defaults, BBBRaid);
	  BBBRaidMarker:SetScale(BBBRaid.scale);
	  BBBRaidMarker:ClearAllPoints();
	  BBBRaidMarker:SetPoint("RIGHT", UIParent, "CENTER");
	  BBBRaidMover:Show();
	  updateShow();
	  showMode();
   elseif (command=="mode") then
	  showMode();
   elseif (mode_list[command] ) then
	  if (arg1 == nil) then
		 showMode();
	  elseif ((arg1 == 'show') or (arg1=='1')) then
		 setDisplay(command, 1);
	  elseif ((arg1 == 'hide') or (arg1=='0')) then
		 setDisplay(command, 0);
	  else
		 print(
			"/|cffff7d0abrm|r solo/party/raid [show||hide]"
			  );
	  end
   else
	  help();
   end
end

function setDisplay(mode, val) 
   BBBRaid["display"][mode] = val;
   if ( val > 0 ) then
	  print("Set "..mode.." => ".."|cff00ff00show|r");
   else
	  print("Set "..mode.." => ".."|cffff3300hide|r");
   end
   
   updateDisplay()
end
